/*
** Module   :PSCOLL.CPP
** Abstract :
**
** Copyright (C) Sergey I. Yevtushenko
** Log: Thu  03/04/97   Designed from two different source trees
*/

#define INCL_WIN
#define INCL_NLS
#define INCL_DOS
#define INCL_GPI
#include <os2.h>
#include <pscoll.h>
#include <string.h>
#include <stdio.h>
#include <dosqss.h>

#define PQ_BUFSIZE  0xFFFF
#define MQ_BUFSIZE  0x8000

char *ppszType[]=
{
    "Sys", /* 00 */
    "VDM", /* 01 */
    "VIO", /* 02 */
    "PM ", /* 03 */
    "Det", /* 04 */
    "Unk"  /*    */
};


PQTOPLEVEL pqData = 0;

APIRET QuerySysInfo(void)
{
    APIRET RC;
    if(!pqData)
    {
        DosAllocMem ((PPVOID)&pqData,
                     PQ_BUFSIZE,
                     PAG_READ | PAG_WRITE | PAG_COMMIT | OBJ_TILE);
    }
    memset(pqData, 0, PQ_BUFSIZE);
    RC = DosQuerySysState(0x1F, 0, 0, 0, pqData, PQ_BUFSIZE);
    return RC;
}

//-------------------------------------------------------

ProcInfo * ProcessCollection::LocatePID(int pid)
{
    for(int i = 0; i < Count(); i++)
    {
        ProcInfo * pInfo = (ProcInfo*)Get(i);
        if(pInfo && pInfo->iPid == pid)
            return pInfo;
    }
    return 0;
}

ProcInfo * ProcessCollection::CheckAndInsert(ProcInfo *pInfo, int& flag)
{
    DWord index = Look(pInfo);
    if(index < Count())
    {
        if(!Compare(pInfo, Get(index))) //------ Process exist ?
        {
            //------ Process already here
            delete pInfo;
            pInfo = (ProcInfo*)Get(index);
            flag = 0;
        }
        else //------ No process don't exist
        {
            Add(pInfo);
            flag = 1; //Additional info should be filled
        }
    }
    else
    {
        Add(pInfo);
        flag = 1;
    }
    return pInfo;
}

void ProcessCollection::CollectData()
{
    APIRET RC;

    RC = QuerySysInfo();
    if (!RC)
    {
        PQPROCESS pProcRecord = pqData->procdata;

        while ( pProcRecord->rectype != 3 )
        {
            PQTHREAD pThread = pProcRecord->threads;
            ProcInfo * pInfo = new ProcInfo;
            int iNeedFill = 0;

            pInfo->iPid = pProcRecord->pid;

//---- Check: if there any record found for that process

            pInfo = CheckAndInsert(pInfo, iNeedFill);

            if(iNeedFill)
            {
                pInfo->cProxName[0] = 0;
                pInfo->iSid = pProcRecord->sessid;
                pInfo->iType = pProcRecord->type;
                pInfo->lOldUser   =
                pInfo->lOldSystem =
                pInfo->lDeltaUser =
                pInfo->lDeltaSystem =
                pInfo->iUsage     = 0;

                DosQueryModuleName(pProcRecord->hndmod, sizeof(pInfo->cProxName), pInfo->cProxName);
                if(!pInfo->cProxName[0])
                    strcpy(pInfo->cProxName, "<zombie>");
                if(pInfo->iType > 5 || pInfo->iType < 0)
                    pInfo->iType = 5;

            }
            if(pThread)
            {
                int j;
                unsigned long ulSys = 0;
                unsigned long ulUsr = 0;

                for(j = 0; j < pProcRecord->threadcnt; j++)
                {
                    ulSys += pThread->systime;
                    ulUsr += pThread->usertime;
                    pThread++;
                }
                pInfo->lDeltaUser   = ulUsr - pInfo->lOldUser;
                pInfo->lDeltaSystem = ulSys - pInfo->lOldSystem;
                pInfo->lOldUser     = ulUsr;
                pInfo->lOldSystem   = ulSys;
            }

//--- Anyway, process are here, touch it

            pInfo->iTouched = 1;

            pProcRecord = (PQPROCESS) (pThread) ;
        }
    }

//------- Ok, data filled. Cleanup database
    ulTotalSys = 0;
    ulTotalUsr = 0;
    for(int i = 0; i < Count(); i++)
    {
        ProcInfo * pInfo = (ProcInfo *)Get(i);
        if(pInfo->iTouched == 0)        // Process not touched last time
        {
            Free(Remove(i));
            i--;
        }
        else
        {
            ulTotalSys += pInfo->lDeltaSystem;
            ulTotalUsr += pInfo->lDeltaUser;
            pInfo->iTouched = 0;
        }
    }
    if((ulTotalSys+ulTotalUsr) <= 0)
    {
        ulTotalUsr = 0;
        ulTotalSys = 1;
    }
}

int ProcessCollection::Pid(int i)
{
    ProcInfo* pInfo = (ProcInfo *)Get(i);
    if(pInfo)
        return pInfo->iPid;
    return 0;
}

int ProcessCollection::CPULoad(int i)
{
    ProcInfo* pInfo = (ProcInfo *)Get(i);
    if(!pInfo)
        return 0;
    return pInfo->iUsage;
}
void ProcessCollection::Print(int i, char* cStr)
{
    ProcInfo* pInfo = (ProcInfo *)Get(i);
    cStr[0] = 0;

    if(!pInfo)
        return;

    int lUser = pInfo->lDeltaUser;

    if(pInfo->iType == 1) // Hack for VDM sessions
        lUser += pInfo->lDeltaSystem;

    pInfo->iUsage = (lUser * 100)/(ulTotalUsr + ulTotalSys);
    if(pInfo->iUsage < 0)
        pInfo->iUsage = 0;

    int usedLen = pInfo->iUsage / 10;

    if((pInfo->iUsage%10) > 5)
        usedLen++;

    int freeLen = 10 - usedLen;
    char cFiller[11];

    if(usedLen)
        memset(cFiller, '', usedLen);
    if(freeLen)
        memset(&cFiller[usedLen], '', freeLen);

    cFiller[10] = 0;

    sprintf(cStr,"%4d %s  %s %3d%% %s",
        pInfo->iPid,
        ppszType[pInfo->iType],
        cFiller,
        pInfo->iUsage,
        pInfo->cProxName
     );
}

